"""Basic file I/O objects for QC calculations."""

from pathlib import Path
from typing import Any, Dict, Union

from typing_extensions import Self

from .abc_io import InputBase, SpecificationBase, SuccessfulResultBase
from .file import File


class GenericFileInput(InputBase):
    """Generic file-based inputs for a calculation."""

    @classmethod
    def from_directory(
        cls,
        *,
        directory: Union[str, Path],
        program: str,
        extras: Dict[str, Any],
    ) -> Self:
        """Create a GenericFileInput from a directory."""
        directory = Path(directory)

        files = {}
        for filepath in directory.glob("*"):
            if filepath.is_file():
                data = filepath.read_bytes()
                try:
                    file = File(data=data.decode("utf-8"))
                except UnicodeDecodeError:
                    file = File(data=data)
                files[filepath.name] = file

        return cls(spec=SpecificationBase(program=program, files=files), extras=extras)

    def to_directory(self, directory: Union[str, Path] = Path(".")) -> None:
        """Write files to a directory."""
        directory = Path(directory)
        directory.mkdir(exist_ok=True)
        for filename, file in self.spec.files.items():
            file.to_disk(directory / filename)


class GenericFileResult(SuccessfulResultBase):
    """Generic result class for pure file based I/O."""

    input_data: GenericFileInput



"""File model for handling string and binary data."""

import json
from base64 import b64decode, b64encode
from pathlib import Path
from typing import Dict, Union

from pydantic import BaseModel, validator
from typing_extensions import Self


class File(BaseModel):
    """Class for managing file data.

    Attributes:
        name: filename.
        files: str or bytes of a file.
    """

    data: Union[str, bytes]

    @validator("data", pre=True)
    def convert_base64_to_bytes(cls, value):
        """Convert base64 encoded data to bytes."""
        if isinstance(value, str) and value.startswith("base64:"):
            b64_encoded_data = value[7:]
            return b64decode(b64_encoded_data)
        return value

    def json(self, *args, **kwargs):
        """Return a JSON representation of the object."""
        return json.dumps(self.dict(*args, **kwargs))

    def dict(self, *args, **kwargs):
        """Return a dict representation of the object encoding bytes as b64 strings."""
        data = self.data
        if isinstance(self.data, bytes):
            data = f"base64:{b64encode(self.data).decode('utf-8')}"
        return {"data": data}

    @classmethod
    def from_disk(cls, filepath: Union[Path, str]) -> Self:
        """Create a File object from a file on disk."""
        filepath = Path(filepath)
        data = filepath.read_bytes()
        try:
            return cls(data=data.decode("utf-8"))  # str
        except UnicodeDecodeError:  # bytes
            return cls(data=data)

    def to_disk(self, filepath: Union[str, Path]) -> None:
        """Write file to disk."""
        filepath = Path(filepath)
        mode = "w" if isinstance(self.data, str) else "wb"
        filepath.open(mode).write(self.data)


class FilesMixin(BaseModel):
    """Mixin for models that have files that need to be serialized to JSON.

    Attributes:
        files: A dict mapping filenames to Files
    """

    files: Dict[str, File]

    def dump_files(self, directory: Union[str, Path] = Path(".")):
        """Write all files to the specified directory"""
        for filename, file in self.files.items():
            file.to_disk(filename)



"""Abstract Base Classes for all QCIO objects."""

from abc import ABC
from pathlib import Path
from typing import Any, Dict, Union

from pydantic import BaseModel
from typing_extensions import Self


class QCIOBaseModel(BaseModel, ABC):
    """Base Model for all QCIO objects.

    Attributes:
        version: The version of the schema.
        extras: Additional information to bundle with the object. Use for schema
            development and scratch space.
    """

    # NOTE: No version for now until we have a stable schema
    # version: ClassVar[Literal["v1"]] = "v1"
    extras: Dict[str, Any] = {}

    @classmethod
    def from_file(cls, filepath: Union[Path, str]) -> Self:
        """Create an object from a filepath."""
        filepath = Path(filepath)
        return cls.parse_raw(filepath.read_text())
        # v2
        # return cls.model_validate_json(filepath.read_text())

    def to_file(self, filepath: Union[Path, str]) -> None:
        """Write an object to disk as json."""
        filepath = Path(filepath)
        filepath.write_text(self.json())
        # v2
        # filepath.write(self.model_dump())



"""Abstract Base Classes for all QCIO objects."""

from typing import Literal, Optional

from .abc_base import QCIOBaseModel
from .file import FilesMixin
from .molecule import Molecule


class SpecificationBase(FilesMixin, QCIOBaseModel):
    """Basic inputs required to power a QC program.

    Attributes:
        program: The name of the program.
    """

    program: str


class InputBase(QCIOBaseModel):
    """Base class for all input classes.

    Attributes:
        molecule: The Molecule for the computation.
    """

    spec: SpecificationBase


class QCIOInputBase(InputBase):
    """Base class for all structured qcio input classes."""

    molecule: Molecule


class ResultBase(FilesMixin, QCIOBaseModel):
    """Base class for all result classes.

    Attributes:
        input_data: The input data for the computation.
        success: A boolean indicator that the operation succeeded or failed. Allows
            programmatic assessment of all results regardless of if they failed or
            succeeded by checking `result.success`.
        stdout: The primary logging output of the program, whether natively standard
            output or a file. Contains a union or stdout and stderr.
    """

    input_data: InputBase
    success: bool
    stdout: Optional[str] = None


class SuccessfulResultBase(ResultBase):
    """Base object for any successful result

    Attributes:
        success: Always True for a successful result.
    """

    success: Literal[True] = True



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qcio']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.10.7,<2.0.0']

setup_kwargs = {
    'name': 'qcio',
    'version': '0.0.2',
    'description': 'A package for structured Quantum Chemistry data.',
    'long_description': '# qcio\n\n[![image](https://img.shields.io/pypi/v/qcio.svg)](https://pypi.python.org/pypi/qcio)\n[![image](https://img.shields.io/pypi/l/qcio.svg)](https://pypi.python.org/pypi/qcio)\n[![image](https://img.shields.io/pypi/pyversions/qcio.svg)](https://pypi.python.org/pypi/qcio)\n[![Actions status](https://github.com/coltonbh/qcio/workflows/Tests/badge.svg)](https://github.com/coltonbh/qcio/actions)\n[![Actions status](https://github.com/coltonbh/qcio/workflows/Basic%20Code%20Quality/badge.svg)](https://github.com/coltonbh/qcio/actions)\n[![Ruff](https://img.shields.io/endpoint?url=https://raw.githubusercontent.com/charliermarsh/ruff/main/assets/badge/v1.json)](https://github.com/charliermarsh/ruff)\n\nA package for structured Quantum Chemistry data.\n\nInspired by [QCElemental](https://github.com/MolSSI/QCElemental). Build for ease of use and rapid development.\n',
    'author': 'Colton Hicks',
    'author_email': 'github@coltonhicks.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
